@@ -136,6 +136,7 @@ typedef struct vmiopd_SM_info {
 #define NV0080_CTRL_GR_INFO_MAX_SIZE_24_03                                      (0x00000037)
 #define NV0080_CTRL_GR_INFO_MAX_SIZE_24_07                                      (0x00000038)
 #define NV0080_CTRL_GR_INFO_MAX_SIZE_29_00                                      (0x0000003A)
+#define NV0080_CTRL_GR_INFO_MAX_SIZE_2C_03                                      (0x0000003B)
 #define NV2080_CTRL_INTERNAL_GR_MAX_ENGINES_1B_04                               8
 #define NV2080_CTRL_INTERNAL_GR_MAX_SM_v1B_05                                   256
 #define NV2080_CTRL_INTERNAL_GR_MAX_SM_v1E_03                                   240
@@ -221,6 +222,7 @@ typedef struct VGPU_BSP_CAPS
 #define NV2080_CTRL_GPU_ECC_UNIT_COUNT_v27_04 (0x0000001F)
 #define NV2080_CTRL_GPU_ECC_UNIT_COUNT_v28_01 (0x00000021)
 #define NV2080_CTRL_GPU_ECC_UNIT_COUNT_v28_08 (0x00000024)
+#define NV2080_CTRL_GPU_ECC_UNIT_COUNT_v2C_02 (0x00000029)
 
 #define NV2080_ENGINE_TYPE_LAST_v18_01      (0x0000002a)
 #define NV2080_ENGINE_TYPE_LAST_v1C_09      (0x00000034)
@@ -283,6 +285,7 @@ typedef struct VGPU_BSP_CAPS
 #define MAX_NVDEC_ENGINES_V1A_07 5
 #define MAX_NVDEC_ENGINES_V25_00 8
 #define NV0080_CTRL_MSENC_CAPS_TBL_SIZE_V25_00 4
+#define NV0080_CTRL_MSENC_CAPS_TBL_SIZE_V2C_07 6
 #define NV0080_CTRL_NVJPG_CAPS_TBL_SIZE_V18_0C 9
 #define NV0080_CTRL_BSP_CAPS_TBL_SIZE_V09_10 8
 #define NV2080_CTRL_GPU_MAX_CONSTRUCTED_FALCONS_V25_01 0x40
@@ -488,14 +491,14 @@ typedef NV2080_CTRL_CE_GET_CAPS_V2_PARAMS VGPU_CE_GET_CAPS_V2[NV2080_ENGINE_TYPE
 
 typedef struct GSP_FIRMWARE GSP_FIRMWARE;
 
-ct_assert(NV2080_CTRL_GPU_ECC_UNIT_COUNT == NV2080_CTRL_GPU_ECC_UNIT_COUNT_v28_08);
+ct_assert(NV2080_CTRL_GPU_ECC_UNIT_COUNT == NV2080_CTRL_GPU_ECC_UNIT_COUNT_v2C_02);
 ct_assert(NV2080_ENGINE_TYPE_LAST == 0x54);
 ct_assert(NV2080_CTRL_BUS_INFO_MAX_LIST_SIZE == NV2080_CTRL_BUS_INFO_MAX_LIST_SIZE_v1C_09);
 ct_assert(NV2080_CTRL_FB_FS_INFO_MAX_QUERIES == NV2080_CTRL_FB_FS_INFO_MAX_QUERIES_v24_00);
 ct_assert(NV2080_CTRL_FB_FS_INFO_MAX_QUERY_SIZE == NV2080_CTRL_FB_FS_INFO_MAX_QUERY_SIZE_v1A_1D);
 ct_assert(NV2080_CTRL_GRMGR_GR_FS_INFO_MAX_QUERIES == NV2080_CTRL_GRMGR_GR_FS_INFO_MAX_QUERIES_v1A_1D);
 ct_assert(NV2080_CTRL_GRMGR_MAX_SMC_IDS == NV2080_CTRL_GRMGR_MAX_SMC_IDS_v1A_1D);
-ct_assert((NV0080_CTRL_GR_INFO_INDEX_MAX + 1) == NV0080_CTRL_GR_INFO_MAX_SIZE_29_00);
+ct_assert((NV0080_CTRL_GR_INFO_INDEX_MAX + 1) == NV0080_CTRL_GR_INFO_MAX_SIZE_2C_03);
 ct_assert(NV2080_CTRL_INTERNAL_GR_MAX_ENGINES == NV2080_CTRL_INTERNAL_GR_MAX_ENGINES_1B_04);
 ct_assert(NV2080_CTRL_GR_SM_ISSUE_RATE_MODIFIER_V2_MAX_LIST_SIZE == NV2080_CTRL_GR_SM_ISSUE_RATE_MODIFIER_V2_MAX_LIST_SIZE_v2B_06);
 ct_assert(NV2080_CTRL_GR_SM_ISSUE_THROTTLE_CTRL_MAX_LIST_SIZE == NV2080_CTRL_GR_SM_ISSUE_THROTTLE_CTRL_MAX_LIST_SIZE_v2B_10);
@@ -531,8 +534,7 @@ ct_assert(NV0000_CTRL_P2P_CAPS_INDEX_TABLE_SIZE == NV0000_CTRL_P2P_CAPS_INDEX_TA
 ct_assert(NV0000_CTRL_SYSTEM_MAX_ATTACHED_GPUS == NV0000_CTRL_SYSTEM_MAX_ATTACHED_GPUS_v21_02);
 ct_assert(VM_UUID_SIZE == VM_UUID_SIZE_v21_02);
 ct_assert(NV2080_CTRL_MAX_PCES == NV2080_CTRL_MAX_PCES_v21_0A);
-// Temporarily disabling this assert due to bug 5381410.
-//ct_assert(NV0080_CTRL_MSENC_CAPS_TBL_SIZE_V25_00 == NV0080_CTRL_MSENC_CAPS_TBL_SIZE);
+ct_assert(NV0080_CTRL_MSENC_CAPS_TBL_SIZE_V2C_07 == NV0080_CTRL_MSENC_CAPS_TBL_SIZE);
 ct_assert(MAX_NVDEC_ENGINES_V1A_07 <= NV2080_CTRL_CMD_INTERNAL_MAX_BSPS);
 ct_assert(MAX_NVDEC_ENGINES_V25_00 == NV2080_CTRL_CMD_INTERNAL_MAX_BSPS);
 ct_assert(NV0080_CTRL_NVJPG_CAPS_TBL_SIZE_V18_0C == NV0080_CTRL_NVJPG_CAPS_TBL_SIZE);
