@@ -84,6 +84,18 @@ timeoutInitializeGpuDefault
         pTD->bScaled = NV_TRUE;
     }
 
+    if (!pTD->bDefaultResetFSMStateTransitionOverridden)
+    {
+        //
+        // Add a default delay. This is a worst case estimate.
+        // A delay of based on HW's suggestion.
+        // See bug 5020859 comment 40 and bug 200636529 comment 20
+        // for the delay calculation.
+        // This delay can be overwritten by regkey RmResetFsmStateTimeoutUs.
+        //
+        pTD->defaultResetFSMStateTransitionUs = gpuGetDefaultResetFSMStateTransitionUs_HAL(pGpu);
+    }
+
     //
     // Note we need to call threadStateResetTimeout() now that the timeout
     // mechanism and values are known to allow threadStateCheckTimeout()
@@ -150,6 +162,19 @@ timeoutRegistryOverride
                   pTD->defaultus);
     }
 
+    if (IS_SIMULATION(pGpu) &&
+        (osReadRegistryDword(pGpu,
+                            NV_REG_STR_RM_RESET_FSM_STATE_TRANSITION_TIMEOUT_US,
+                            &data32) == NV_OK) &&
+        (data32 > gpuGetDefaultResetFSMStateTransitionUs_HAL(pGpu)))
+    {
+        // The default delay value can only be overwritten by a greater value
+        pTD->defaultResetFSMStateTransitionUs = data32;
+        pTD->bDefaultResetFSMStateTransitionOverridden = NV_TRUE;
+        NV_PRINTF(LEVEL_ERROR, "Overriding default timeout for reset FSM state transition to 0x%08x\n",
+                  pTD->defaultResetFSMStateTransitionUs);
+    }
+
     // Override timeout flag values
     if (osReadRegistryDword(pGpu,
                             NV_REG_STR_RM_OVERRIDE_DEFAULT_TIMEOUT_FLAGS,
@@ -210,9 +235,10 @@ timeoutSet
     NvU32         flags
 )
 {
-    OBJTMR             *pTmr;
-    NvU64               timeInNs;
-    NvU64               timeoutNs;
+    OBJTMR  *pTmr;
+    NvU64   timeInNs;
+    NvU64   timeoutNs;
+    OBJGPU  *pGpu = pTD->pGpu;
 
     portMemSet(pTimeout, 0, sizeof(*pTimeout));
 
@@ -252,8 +278,15 @@ timeoutSet
 
     // Set end time for elapsed time methods
     timeoutNs = (NvU64)timeoutUs * 1000;
+
     if (pTimeout->flags & GPU_TIMEOUT_FLAGS_OSTIMER)
     {
+        // WAR for FMODEL: Increase timeout by 10x to handle slow GSPRM on FMODEL on CPURM
+        if (IS_GSP_CLIENT(pGpu) && IS_FMODEL(pGpu))
+        {
+            timeoutNs *= 10;
+        }
+
         //
         // For small timeouts (timeout durations on the order of magnitude of
         // the OS tick resolution), starting the timeout near the end of a tick
@@ -271,7 +304,6 @@ timeoutSet
     else if ((pTimeout->flags & GPU_TIMEOUT_FLAGS_TMR) ||
              (pTimeout->flags & GPU_TIMEOUT_FLAGS_TMRDELAY))
     {
-        OBJGPU *pGpu = pTD->pGpu;
         NV_ASSERT_OR_RETURN_VOID(pGpu != NULL);
 
         OBJGPU *pParentGpu = gpumgrGetParentGPU(pGpu);
